import { ExistingProvider, InjectionToken } from '@angular/core';
import * as i0 from "@angular/core";
/** A token used to provide the native document implementation for `DocumentRef`. By default, `CoreModule` will provide angular's DOCUMENT token. */
export declare const DOCUMENT: InjectionToken<Document>;
/**
 * Provides an injectable wrapper for the `document` object.
 * Inject this in your services/components and you will be able to easily mock or spy on the native `document` object in your tests.
 *
 * By default, the `nativeDocument` property will point to angular's DOM adapter, thus facilitating DOM access and manipulation
 * on the different platforms.
 * To mock the native document, provide a value for the `DOCUMENT` token from `@bespunky/angular-zen/core`.
 * You will safely mock it without trashing angular's `DOCUMENT` provider.
 *
 * @see document-ref.service.spec.ts for examples.
 */
export declare class DocumentRef {
    readonly nativeDocument: any;
    /**
     * Creates an instance of `DocumentRef`.
     *
     * @param {*} nativeDocument The native document provided by the `DOCUMENT` token of `@bespunky/angular-zen/core`. See `DocumentRef` for details.
     */
    constructor(nativeDocument: any);
    static ɵfac: i0.ɵɵFactoryDeclaration<DocumentRef, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<DocumentRef>;
}
/**
 * The default provider for the `DOCUMENT` token. Uses angular's DOM adapters which will be injected according to the platform.
 */
export declare const DocumentProvider: ExistingProvider;
/**
 * A bundle of all providers needed for DocumentRef to work.
 */
export declare const DocumentRefProviders: ExistingProvider[];
