import { PartialObserver } from 'rxjs';
import { EmbeddedViewRef } from '@angular/core';
import { OnObserverContext } from './on-observer-context';
import { ViewRenderCommitment } from './view-render-commitment';
/** Represents the name of an observer call. */
export declare type ObserverName = 'resolving' | Exclude<keyof PartialObserver<unknown>, 'closed'>;
/** Represents supported duration units. */
export declare type DurationUnit = 'ms' | 's' | 'm';
/** Represents duration as a number, which will be treated as milliseconds, or a string with the format of `<number><ms | s | ms>`.
 * Numbers can be either integers or floats.
 * For example:
 * - `3000` - 3 seconds.
 * - `'10s'` - 10 seconds.
 * - `'0.5m'` - 30 seconds.
 * - `'100ms'` - 100 milliseconds.
 */
export declare type DurationAnnotation = number | `${number}${DurationUnit}`;
/** Represents a duration, broken into its components. */
export declare type DurationBreakdown = Record<DurationUnit, number> & Record<`total${'Milliseconds' | 'Seconds' | 'Minutes'}`, number>;
/** Represents a view rendered by an {@link OnObserverBaseDirective `*onObserver`} directive. */
export declare type RenderedView<T> = EmbeddedViewRef<OnObserverContext<T>>;
/** Represents a map of render commitments holding the information about what views to render and when. */
export declare type RenderCommitmentMap<T> = Map<string, ViewRenderCommitment<T>>;
/** Represents the supported view modes for {@link OnObserverBaseDirective `*onObserver`} directives. */
export declare type ViewMode = 'multiple' | 'single';
