import { InjectionToken, FactoryProvider } from '@angular/core';
import * as i0 from "@angular/core";
/**
 * An injectable token that will allow us to replace the provider for the native window object when necessary (e.g. mocking the `window` object).
 */
export declare const WINDOW: InjectionToken<Window>;
/**
 * Provides an injectable wrapper for the `window` object.
 *
 * Inject this in your services/components and you will be able to easily mock or spy on the native `window` object in your tests.
 * You can replace the default `WINDOW` token provider, which allows you to mock the `window` object.
 *
 * @see window-ref.service.spec.ts for examples.
 */
export declare class WindowRef {
    readonly nativeWindow: any;
    /**
     * Creates an instance of WindowRef.
     *
     * @param {*} nativeWindow The native window provided by the `WINDOW` token of `@bespunky/angular-zen/core`. See `WindowRef` for details.
     */
    constructor(nativeWindow: any);
    static ɵfac: i0.ɵɵFactoryDeclaration<WindowRef, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<WindowRef>;
}
/**
 * Provides a platform dependant implementation for retrieving the `window` object.
 *
 * @returns `window` for browser platforms and a new object for non-browser platforms.
 */
export declare function windowFactory(platformId: any): Window | Object;
/**
 * The default provider for the `WINDOW` token. Provides `window` for browser platforms and a new object for non-browser platforms.
 */
export declare const WindowProvider: FactoryProvider;
/**
 * A bundle of all providers needed for WindowRef to work.
 */
export declare const WindowRefProviders: FactoryProvider[];
