import { ClassProvider, FactoryProvider, Provider } from '@angular/core';
import { UrlLocalizationConfig } from './url-localization-config';
/**
 * The default configuration for url localization when loading the language integration module.
 * Uses the `NoopUrlLocalizer` as strategy and does not force https. Localization and delocalization will always return an unchanged url url.
 */
export declare const DefaultUrlLocalizationConfig: UrlLocalizationConfig;
/**
 * Creates the appropriate DI compatible provider for the `UrlLocalizer` class, depending on the strategy specified in the url localization configuration.
 * If the configured strategy is a number, `RoutePositionUrlLocalizer` will be used.
 * If the configured strategy is a string, `QueryParamsUrlLocalizer` will be used.
 * If the configured strategy is a valid `UrlLocalizer` provider, the provider will be used as is.
 * Otherwise, `NoopUrlLocalizer` will be used.
 *
 * @export
 * @param {UrlLocalizationConfig} config The url localization configuration holding the strategy.
 * @returns {(ClassProvider | FactoryProvider)} A DI compatible provider for the `UrlLocalizer` class with the implementation appropriate for the specified strategy.
 */
export declare function provideUrlLocalizer(config: UrlLocalizationConfig): ClassProvider | FactoryProvider;
/**
 * Creates the providers for the `UrlLocalization` token and the `UrlLocalizer` class.
 *
 * @export
 * @param {UrlLocalizationConfig} [config] (Optional) The configuration for url localization tools. Default is `DefaultUrlLocalizationConfig`.
 * @returns {Provider[]} The providers for the `UrlLocalization` token and the `UrlLocalizer` class.
 */
export declare function provideUrlLocalization(config?: UrlLocalizationConfig): Provider[];
